use std::fmt::Display;
use std::sync::Arc;

use anyhow::Result;
use libpt::log::warn;
use minijinja::context;
use minijinja::Environment;
use serde;
use thiserror::Error;
use warp::reject;
use warp::reply::Reply;
use warp::reply::Response;
use warp::Filter;

use crate::config::Config;
use crate::vault::VaultRef;

#[derive(Clone)]
pub struct Service<'tp> {
    vault: VaultRef,
    config: Config,
    env: Environment<'tp>,
}
impl<'tp> Service<'tp> {
    fn new(vault: VaultRef, config: Config, env: Environment<'tp>) -> Arc<Self> {
        Self { vault, config, env }.into()
    }
}

pub async fn serve(vault: VaultRef, config: Config) -> Result<()> {
    let mut env = Environment::new();
    env.add_template("index", include_str!("../../data/www/admin.html"))?;

    let service = Service::new(vault, config, env);
    let service2 = service.clone();

    let routes = warp::path::end()
        .map(move || service2.clone())
        .and_then(overview);

    warp::serve(routes)
        .run(service.config.addr_admin.unwrap())
        .await;

    warn!("exited the admin interface");
    Ok(())
}

#[derive(Debug, Error)]
struct TemplateError(#[from] minijinja::Error);

impl reject::Reject for TemplateError {}
impl Display for TemplateError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "something went wrong with producing this page")
    }
}

async fn overview(serv: Arc<Service<'_>>) -> Result<Box<dyn warp::Reply>, warp::Rejection> {
    let r = Response::new(
        serv.env
            .get_template("index")
            .map_err(TemplateError::from)?
            .render(context!(TITLE => "Wooly-Vault", AUTHOR => env!("CARGO_PKG_AUTHORS")))
            .map_err(TemplateError::from)?
            .into(),
    );

    Ok(Box::new(r))
}

async fn styles() -> Result<Box<dyn warp::Reply>, warp::Rejection> {
    let r = include_str!("../../data/www/styles.css").to_string();
    Ok(Box::new(r))
}
